package org.example.wedding_app_backend.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.convert.converter.Converter;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.oauth2.server.resource.authentication.JwtAuthenticationConverter;

import java.util.Collection;
import java.util.Map;
import java.util.stream.Collectors;

@Configuration
public class JwtConfig {

    private final String principleAttribute;

    private final String resourceId;

    public JwtConfig(
            @Value("${jwt.auth.converter.principle-attribute}") String principleAttribute,
            @Value("${jwt.auth.converter.resource-id}") String resourceId) {
        this.principleAttribute = principleAttribute;
        this.resourceId = resourceId;
    }


    @Bean
    public JwtAuthenticationConverter jwtAuthenticationConverterForKeycloak() {
        Converter<Jwt, Collection<GrantedAuthority>> jwtGrantedAuthoritiesConverter = jwt -> {

            Map<String, Map<String, Collection<String>>> realmAccess = jwt.getClaim("resource_access");
            Collection<String> roles = realmAccess.get(resourceId).get("roles");
            return roles.stream()
                    .map(role -> new SimpleGrantedAuthority("ROLE_" + role.toUpperCase()))
                    .collect(Collectors.toList());
        };

        var jwtAuthenticationConverter = new JwtAuthenticationConverter();
        jwtAuthenticationConverter.setJwtGrantedAuthoritiesConverter(jwtGrantedAuthoritiesConverter);

        jwtAuthenticationConverter.setPrincipalClaimName(principleAttribute);

        return jwtAuthenticationConverter;
    }

}
