package org.example.wedding_app_backend.wedding.controller;

import lombok.RequiredArgsConstructor;
import org.example.wedding_app_backend.wedding.dto.GetWeddingResponse;
import org.example.wedding_app_backend.wedding.function.WeddingMapper;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.service.WeddingService;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.net.Authenticator;
import java.util.UUID;

@RestController
@RequiredArgsConstructor
public class WeddingController {

    private final WeddingService weddingService;

    private final WeddingMapper weddingMapper;

    @GetMapping("/api/weddings/{weddingId}")
    public ResponseEntity<GetWeddingResponse> getWedding(@PathVariable UUID weddingId) {
        return weddingService.findWedding(weddingId)
                .map(weddingMapper::entityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }
    
    @GetMapping(path = "/api/weddings/{weddingId}/background", produces = MediaType.IMAGE_JPEG_VALUE)
    public ResponseEntity<byte[]> getWeddingImage(@PathVariable("weddingId") UUID weddingId) {
        return weddingService.findWedding(weddingId)
                .map(WeddingEntity::getBackgroundImage)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());

    }

    @PostMapping(path = "/api/users/background", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @ResponseStatus(HttpStatus.CREATED)
    public void putWeddingImage(@AuthenticationPrincipal Jwt jwt, @RequestParam("image") MultipartFile request) {
        weddingService.saveWeddingBackgroundImage(UUID.fromString(jwt.getSubject()), request);
    }
}
