package org.example.wedding_app_backend.wedding.service;

import lombok.RequiredArgsConstructor;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

@Service
@RequiredArgsConstructor
public class WeddingService {

    private final WeddingRepository weddingRepository;

    public Optional<WeddingEntity> findWedding(UUID id) {
        return weddingRepository.findById(id);
    }

    public void saveWedding(WeddingEntity wedding) {
        weddingRepository.save(wedding);
    }

    public Optional<WeddingEntity> findWeddingByUser(UUID id) {
        return weddingRepository.findByUserId(id);
    }

    public void saveWeddingBackgroundImage (UUID id, MultipartFile request) {
        weddingRepository.findByUserId(id)
                .ifPresentOrElse(wedding -> {
                        try {
                            wedding.setBackgroundImage(request.getBytes());
                            weddingRepository.save(wedding);
                        } catch (IOException e) {
                            throw new RuntimeException(e);
                        }
                    },
                    () -> {
                        throw new ResponseStatusException(HttpStatus.NOT_FOUND);
                    }
                );
    }
}
