package org.example.wedding_app_backend.weddingGuest.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.java.Log;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.service.WeddingService;
import org.example.wedding_app_backend.weddingConfirmationGroup.service.WeddingConfirmationGroupService;
import org.example.wedding_app_backend.weddingGuest.dto.*;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.function.GetGuestInformationWithGroup;
import org.example.wedding_app_backend.weddingGuest.function.WeddingGuestMapper;
import org.example.wedding_app_backend.weddingGuest.service.WeddingGuestService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;
import java.util.*;

@RestController
@Log
@RequiredArgsConstructor
public class WeddingGuestController {

    private final WeddingGuestService weddingGuestService;
    private final WeddingConfirmationGroupService weddingConfirmationGroupService;
    private final GetGuestInformationWithGroup getGuestInformationWithGroup;

    private final WeddingGuestMapper weddingGuestMapper;
    private final WeddingService weddingService;

    //Quiz handling

    @GetMapping("/api/weddings/{weddingId}/{guestId}/quiz/state")
    public ResponseEntity<GetWeddingGuestQuizState> getWeddingGuestQuizState(@PathVariable("weddingId") UUID weddingId,@PathVariable("guestId") UUID guestId) {
        return weddingGuestService.findWeddingGuestById(weddingId,guestId)
                .map(weddingGuestMapper::quizEntityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @GetMapping("/api/weddings/{weddingId}/{guestId}/quiz/score")
    public ResponseEntity<GetWeddingGuestQuizScore> getWeddingGuestQuizScore(@PathVariable("weddingId") UUID weddingId, @PathVariable("guestId") UUID guestId) {
        return weddingGuestService.findWeddingGuestById(weddingId,guestId)
                .map(weddingGuestMapper::quizScoreEntityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    @GetMapping("/api/weddings/{weddingId}/{guestId}/games/hall")
    public ResponseEntity<List<GetWeddingGuestsFinalScores>> getWeddingQuizScores(@PathVariable("weddingId") UUID weddingId, @PathVariable("guestId") UUID guestId) {
        return weddingService.findWedding(weddingId)
                .map(wedding->weddingGuestService.findAllWeddingGuestsThatFinishedQuiz(weddingId)
                        .map(weddingGuestMapper::quizScoresEntityToRepresentation)
                        .map(ResponseEntity::ok)
                        .orElse(ResponseEntity.notFound().build()))
                .orElse(ResponseEntity.notFound().build());
    }

    //

    @GetMapping("/api/wedding/{weddingId}/{guestId}/games/who-knows-better/state")
    public ResponseEntity<GetWeddingGuestWhoKnowsBetterQuizState> getWhoKnowsBetterQuizState(@PathVariable("weddingId") UUID weddingId, @PathVariable("guestId") UUID guestId) {
        return weddingGuestService.findWeddingGuestById(weddingId,guestId)
                .map(weddingGuestMapper::whoKnowsBetterQuizEntityToRepresentation)
                .map(ResponseEntity::ok)
                .orElse(ResponseEntity.notFound().build());
    }

    //Guest handling

    @GetMapping("/api/users/guests")
    public ResponseEntity<List<GetWeddingGuestWithGroupInformation>> getWeddingGuestsData(@AuthenticationPrincipal Jwt jwt) {

        //UUID.fromString(jwt.getSubject())
        return weddingService.findWeddingByUser(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .map(wedding->weddingGuestService.findAllByWeddingId(wedding.getId())
                        .map(getGuestInformationWithGroup)
                        .map(ResponseEntity::ok)
                        .orElse(ResponseEntity.notFound().build()))
                .orElse(ResponseEntity.status(HttpStatus.NOT_FOUND).build());
    }

    @PatchMapping(path = "/api/users/guests")
    public void updateWeddingGuestParametersModifiableByUser(@RequestBody GetWeddingGuestWithGroupInformation request)
    {
        weddingGuestService.findWeddingGuest(request.getId())
                .ifPresentOrElse(
                        guest -> {
                            try {
                                weddingGuestService.updateGuestParametersModifiableByUser(guest,request);
                            } catch (IOException ex) {
                                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR);
                            }
                        },
                        () -> {
                            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
                        }
                );
    }

    @PostMapping(path = "/api/users/guests")
    public void createWeddingGuest(@AuthenticationPrincipal Jwt jwt,@RequestBody GetWeddingGuestWithGroupInformation request)
    {

        //UUID.fromString(jwt.getSubject())
        weddingService.findWeddingByUser(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .ifPresentOrElse(
                        wedding -> weddingConfirmationGroupService.findWeddingConfirmationGroupById(request.getWeddingConfirmationGroup().getId())
                                .ifPresentOrElse(
                                        group -> {
                                            try {
                                                weddingGuestService.createWeddingGuest(request, wedding, group);
                                            } catch (IOException ex) {
                                                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR);
                                            }
                                        },
                                        () -> { throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Group not found"); }
                                ),
                        () -> { throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Wedding not found"); }
                );
    }

    @DeleteMapping(path = "/api/users/guests")
    public void deleteWeddingGuest(@RequestBody GetWeddingGuestWithGroupInformation request)
    {
        weddingGuestService.findWeddingGuest(request.getId())
                .ifPresentOrElse(
                        guest -> {
                            try {
                                weddingGuestService.deleteGuest(guest);
                            } catch (IOException ex) {
                                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR);
                            }
                        },
                        () -> {
                            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
                        }
                );
    }

    // TAK POWINO BYC
    @PatchMapping(path = "/api/wedding/{weddingId}/guests/{guestId}")
    public void updateWeddingGuestPresenceConfirmation(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId,
            @RequestParam("presenceConfirmed") boolean presenceConfirmed)
    {
        weddingGuestService.findWeddingGuestById(weddingId, guestId)
                .ifPresentOrElse(
                        guest -> {
                            try {
                                weddingGuestService.updateGuestPresenceConfirmation(guest, presenceConfirmed);
                            } catch (IOException ex) {
                                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR);
                            }
                        },
                        () -> {
                            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
                        }
                );
    }

}
