package org.example.wedding_app_backend.weddingGuest.service;

import lombok.RequiredArgsConstructor;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingConfirmationGroup.entity.WeddingConfirmationGroupEntity;
import org.example.wedding_app_backend.weddingGuest.dto.GetWeddingGuestWithGroupInformation;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.repository.WeddingImageRepository;
import org.example.wedding_app_backend.weddingQuiz.entity.WeddingQuizEntity;
import org.example.wedding_app_backend.weddingQuiz.repository.WeddingQuizRepository;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.model.WhoKnowsBetterQuiz;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.repository.WhoKnowsBetterQuizRepository;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.*;

@Service
@RequiredArgsConstructor
public class WeddingGuestService {

    private final WeddingGuestRepository weddingGuestRepository;

    private final WeddingRepository weddingRepository;

    private final WeddingQuizRepository weddingQuizRepository;

    private final WhoKnowsBetterQuizRepository whoKnowsBetterQuizRepository;

    public void save(WeddingGuestEntity weddingGuest) {weddingGuestRepository.save(weddingGuest);}

    public Optional<WeddingGuestEntity> findWeddingGuestById(UUID weddingId, UUID guestId) {return weddingGuestRepository.findByWeddingIdAndId(weddingId, guestId);}

    public Optional<List<WeddingGuestEntity>> findAllByWeddingId(UUID weddingId) {return weddingGuestRepository.findAllByWeddingId(weddingId);}

    public Optional<WeddingGuestEntity> findWeddingGuest(UUID id) {
        return weddingGuestRepository.findById(id);
    }

    public Optional<List<WeddingGuestEntity>> findAllWeddingGuestsThatFinishedQuiz(UUID id){
        return weddingGuestRepository.findAllByWeddingIdAndQuizFinishedTrue(id);
    }

    public void updateQuizPoints(WeddingGuestEntity guest, int score) throws IOException {
        guest.setQuizPoints(guest.getQuizPoints() + score);

        try {
            weddingGuestRepository.save(guest);
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    public void updateQuizIndex(WeddingGuestEntity guest,UUID weddingId) throws IOException{
        guest.setQuestionIndex(guest.getQuestionIndex() + 1);

        weddingQuizRepository.findQuestionById(guest.getQuestionIndex(),weddingId,guest.getQuizSeed())
                .ifPresentOrElse(
                        question -> {
                            //there are still questions
                        },
                        () -> guest.setQuizFinished(true)
                );

        try {
            weddingGuestRepository.save(guest);
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    public void updateWhoKnowsBetterQuizIndex(UUID WeddingGuestId, UUID WeddingId) {

        WeddingGuestEntity guest = weddingGuestRepository.findById(WeddingGuestId)
                .orElseThrow(() -> new NoSuchElementException("Guest with ID " + WeddingGuestId + " not found."));

        guest.setWhoKnowsQuestionIndex(guest.getWhoKnowsQuestionIndex() + 1);

        Optional<WhoKnowsBetterQuiz> nextQuestion = whoKnowsBetterQuizRepository.findByWeddingIdAndQuestionIndex(WeddingId, guest.getWhoKnowsQuestionIndex());

        if(nextQuestion.isEmpty()){
            guest.setWhoKnowsQuizFinished(true);
        }

        weddingGuestRepository.save(guest);
    }

    public void updateGuestPresenceConfirmation(WeddingGuestEntity guest, Boolean presenceConfirmed) throws IOException {
        guest.setPresenceConfirmed(presenceConfirmed);

        try {
            weddingGuestRepository.save(guest);
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    public void createWeddingGuest(GetWeddingGuestWithGroupInformation request,WeddingEntity weddingId,WeddingConfirmationGroupEntity group)throws IOException {
        WeddingGuestEntity guest = new WeddingGuestEntity();

        guest.setId(UUID.randomUUID());
        guest.setPresenceConfirmed(null);
        guest.setQuestionIndex(0);
        guest.setQuizPoints(0);
        guest.setQuizFinished(false);
        guest.setQuizSeed((int) (Math.random()*3)+3);
        guest.setGuestName(request.getGuestName());
        guest.setGuestAge(request.getGuestAge());
        guest.setWhoKnowsQuestionIndex(0);
        guest.setWhoKnowsQuizFinished(false);
        guest.setWeddingConfirmationGroup(group);
        guest.setWedding(weddingId);

        try {
            weddingGuestRepository.save(guest);
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    public void updateGuestParametersModifiableByUser(WeddingGuestEntity guest,GetWeddingGuestWithGroupInformation request) throws IOException{
        guest.setGuestName(request.getGuestName());
        guest.setGuestAge(request.getGuestAge());

        try {
            weddingGuestRepository.save(guest);
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    public void deleteGuest(WeddingGuestEntity guest) throws IOException {
        try {
            weddingGuestRepository.delete(guest);
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

}
