package org.example.wedding_app_backend.weddingImage.service;

import jakarta.persistence.EntityNotFoundException;
import lombok.RequiredArgsConstructor;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.weddingImage.dto.GetWeddingImageResponse;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.repository.WeddingImageRepository;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class WeddingImageService {

    private final WeddingImageRepository weddingImageRepository;

    private final WeddingRepository weddingRepository;

    private final WeddingGuestRepository weddingGuestRepository;

    public Optional<WeddingImageEntity> findWeddingImage(UUID id) {
        return weddingImageRepository.findById(id);
    }

    public void saveWeddingImage(WeddingImageEntity weddingImage) {
        weddingImageRepository.save(weddingImage);
    }

    public Optional<List<GetWeddingImageResponse>> findAllWeddingImages(UUID weddingId, UUID guestId, String sort) {
        return weddingRepository.findById(weddingId)
                .map(wedding -> weddingImageRepository.findAllByWedding(wedding, Sort.by(Sort.Order.desc(sort))))
                .map(images ->
                        images.stream()
                                .map(image -> {
                                    boolean liked = image.getLikedByGuest().stream()
                                            .anyMatch(guest -> guest.getId().equals(guestId));

                                    return GetWeddingImageResponse.builder()
                                            .id(image.getId())
                                            .likes(image.getLikes())
                                            .date(image.getDate().toString())
                                            .liked(liked)
                                            .build();
                                })
                                .collect(Collectors.toList())
                );
    }

    public void updateLikes(UUID guestId, UUID weddingImageId, boolean liked) {
        WeddingGuestEntity guest = weddingGuestRepository.findById(guestId)
                .orElseThrow(() -> new EntityNotFoundException("Guest not found with ID: " + guestId));

        WeddingImageEntity image = weddingImageRepository.findById(weddingImageId)
                .orElseThrow(() -> new EntityNotFoundException("Image not found with ID: " + weddingImageId));

        if (liked) {
            if (!guest.getLikedImages().contains(image)) {
                guest.getLikedImages().add(image);
                image.getLikedByGuest().add(guest);
                image.setLikes(image.getLikes() + 1);
            }
        } else {
            if (guest.getLikedImages().contains(image)) {
                guest.getLikedImages().remove(image);
                image.getLikedByGuest().remove(guest);
                image.setLikes(image.getLikes() - 1);
            }
        }

        weddingGuestRepository.save(guest);
        weddingImageRepository.save(image);
    }
}
