package org.example.wedding_app_backend.weddingPresent.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.java.Log;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.service.WeddingService;
import org.example.wedding_app_backend.weddingGuest.dto.GetWeddingGuestWithGroupInformation;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.service.WeddingGuestService;
import org.example.wedding_app_backend.weddingPresent.dto.GetWeddingPresentResponse;
import org.example.wedding_app_backend.weddingPresent.entity.WeddingPresentEntity;
import org.example.wedding_app_backend.weddingPresent.function.WeddingPresentMapper;
import org.example.wedding_app_backend.weddingPresent.service.WeddingPresentService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;
import java.util.*;

@RestController
@Log
@RequiredArgsConstructor
public class WeddingPresentController {

    private final WeddingPresentService weddingPresentService;

    private final WeddingPresentMapper weddingPresentMapper;

    private final WeddingService weddingService;

    //czesc gosci

    @GetMapping("/api/weddings/{weddingId}/{guestId}/presents")
    public ResponseEntity<List<GetWeddingPresentResponse>> getWeddingPresents(@PathVariable("weddingId") UUID weddingId, @PathVariable String guestId) {
        return weddingService.findWedding(weddingId)
                .map(wedding->weddingPresentService.findAllByWeddingId(weddingId)
                        .map(weddingPresentMapper::entitiesToRepresentation)
                        .map(ResponseEntity::ok)
                        .orElse(ResponseEntity.notFound().build()))
                .orElse(ResponseEntity.notFound().build());
    }

    @PatchMapping("/api/weddings/{weddingId}/{guestId}/presents")
    public void updateWeddingPresentReservedBy(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId,
            @RequestBody GetWeddingPresentResponse request)
    {
        weddingPresentService.findByWeddingIdAndPresentId(weddingId,request.getId())
                .ifPresentOrElse(
                        present -> {
                            try {
                                weddingPresentService.updatePresentReservedBy(present,request);
                            } catch (IOException ex) {
                                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR);
                            }
                        },
                        () -> {
                            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
                        }
                );
    }

    //user site

    @GetMapping("/api/users/presents")
    public ResponseEntity<List<GetWeddingPresentResponse>> getWeddingPresents(@AuthenticationPrincipal Jwt jwt) {

        //UUID.fromString(jwt.getSubject()) narazie tak bo kazdy z nas ma inne
        return weddingService.findWeddingByUser(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .map(wedding->weddingPresentService.findAllByWeddingId(wedding.getId())
                        .map(weddingPresentMapper::entitiesToRepresentation)
                        .map(ResponseEntity::ok)
                        .orElse(ResponseEntity.notFound().build()))
                .orElse(ResponseEntity.status(HttpStatus.NOT_FOUND).build());
    }

    @PatchMapping(path = "/api/users/presents")
    public void updateWeddingPresentParametersModifiableByUser(@RequestBody GetWeddingPresentResponse request)
    {
        weddingPresentService.findById(request.getId())
                .ifPresentOrElse(
                        present -> {
                            try {
                                weddingPresentService.updatePresentParametersModifiableByUser(present,request);
                            } catch (IOException ex) {
                                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR);
                            }
                        },
                        () -> {
                            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
                        }
                );
    }

    @PostMapping(path = "/api/users/presents")
    public void createWeddingPresent(@AuthenticationPrincipal Jwt jwt, @RequestBody GetWeddingPresentResponse request) {

        //UUID.fromString(jwt.getSubject())
        weddingService.findWeddingByUser(UUID.fromString("123e4567-e89b-12d3-a456-426614174099"))
                .ifPresentOrElse(
                        wedding -> {
                            try {
                                weddingPresentService.createWeddingPresent(request, wedding);
                            } catch (IOException ex) {
                                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR);
                            }
                        },
                        () -> { throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Wedding not found"); }
                );
    }

    @DeleteMapping(path = "/api/users/presents")
    public void deleteWeddingGuest(@RequestBody GetWeddingPresentResponse request)
    {
        weddingPresentService.findById(request.getId())
                .ifPresentOrElse(
                        present -> {
                            try {
                                weddingPresentService.deletePresent(present);
                            } catch (IOException ex) {
                                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR);
                            }
                        },
                        () -> {
                            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
                        }
                );
    }

}
