package org.example.wedding_app_backend.weddingPresent.service;

import jakarta.transaction.Transactional;
import lombok.RequiredArgsConstructor;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingConfirmationGroup.entity.WeddingConfirmationGroupEntity;
import org.example.wedding_app_backend.weddingGuest.dto.GetWeddingGuestWithGroupInformation;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingPresent.dto.GetWeddingPresentResponse;
import org.example.wedding_app_backend.weddingPresent.entity.WeddingPresentEntity;
import org.example.wedding_app_backend.weddingPresent.repository.WeddingPresentRepository;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.*;

@Service
@RequiredArgsConstructor
public class WeddingPresentService {

    private final WeddingPresentRepository weddingPresentRepository;

    private final WeddingRepository weddingRepository;

    public void save(WeddingPresentEntity weddingPresentEntity) {
        weddingPresentRepository.save(weddingPresentEntity);
    }

    public Optional<List<WeddingPresentEntity>> findAllByWeddingId(UUID weddingId) {
        return weddingPresentRepository.findAllByWeddingId(weddingId);
    }

    public Optional<WeddingPresentEntity> findByWeddingIdAndPresentId(UUID weddingId,UUID presentId) {return weddingPresentRepository.findByWeddingIdAndId(weddingId,presentId);}

    public Optional<WeddingPresentEntity> findById(UUID presentId) {return weddingPresentRepository.findById(presentId);}

    public void updatePresentReservedBy(WeddingPresentEntity present,GetWeddingPresentResponse response) throws IOException {
        present.setReservedBy(response.getReservedBy());

        try {
            weddingPresentRepository.save(present);
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    public void updatePresentParametersModifiableByUser(WeddingPresentEntity present, GetWeddingPresentResponse request) throws IOException{
        present.setName(request.getName());
        present.setDescription(request.getDescription());
        present.setLinkToPresent(request.getLinkToPresent());

        try {
            weddingPresentRepository.save(present);
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    public void createWeddingPresent(GetWeddingPresentResponse request, WeddingEntity wedding)throws IOException {
        WeddingPresentEntity present = new WeddingPresentEntity();

        present.setId(UUID.randomUUID());
        present.setName(request.getName());
        present.setDescription(request.getDescription());
        present.setLinkToPresent(request.getLinkToPresent());
        present.setWedding(wedding);

        try {
            weddingPresentRepository.save(present);
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    public void deletePresent(WeddingPresentEntity present) throws IOException {
        try {
            weddingPresentRepository.delete(present);
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }


}
