package org.example.wedding_app_backend.weddingQuiz.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.java.Log;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.service.WeddingGuestService;
import org.example.wedding_app_backend.weddingQuiz.dto.GetWeddingQuizResponse;
import org.example.wedding_app_backend.weddingQuiz.function.WeddingQuizQuestionMapper;
import org.example.wedding_app_backend.weddingQuiz.service.WeddingQuizService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;

@RequestMapping("/api/weddings")
@RestController
@Log
@RequiredArgsConstructor
public class WeddingQuizController {

    private final WeddingQuizService weddingQuizService;

    private final WeddingQuizQuestionMapper weddingQuizQuestionMapper;

    private final WeddingGuestService weddingGuestService;



    @GetMapping("{weddingId}/{guestId}/games/quiz")
    public ResponseEntity<GetWeddingQuizResponse> getWeddingQuestion(@PathVariable("weddingId") UUID weddingId, @PathVariable("guestId") UUID guestId) {
        return weddingGuestService.findWeddingGuestById(weddingId,guestId)
                .map(guest->weddingQuizService.findQuestionByGuestId(guest,weddingId)
                        .map(weddingQuizQuestionMapper::entityToRepresentation)
                        .map(ResponseEntity::ok)
                        .orElse(ResponseEntity.notFound().build()))
                .orElse(ResponseEntity.notFound().build());
    }

    @PatchMapping(path = "{weddingId}/{guestId}/games/quiz")
    public void updateWeddingGuestPoints(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId,
            @RequestBody Map<String, Integer> payload)
    {
        weddingGuestService.findWeddingGuest(guestId)
                .ifPresentOrElse(guest -> {
                            try {
                                weddingGuestService.updateQuizPoints(guest, payload.get("score"));
                                weddingGuestService.updateQuizIndex(guest,weddingId);
                            } catch (IOException ex) {
                                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR);
                            }
                        },
                        () -> {
                            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
                        });

    }


}
