package org.example.wedding_app_backend.whoKnowsBetterResults.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.java.Log;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.model.WhoKnowsBetterQuiz;
import org.example.wedding_app_backend.whoKnowsBetterResults.dto.GetAllResults;
import org.example.wedding_app_backend.whoKnowsBetterResults.function.ResultsMapper;
import org.example.wedding_app_backend.whoKnowsBetterResults.service.ResultsService;
import org.springframework.http.ResponseEntity;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.UUID;

@RequestMapping("/api/wedding/{weddingId}/{guestId}/games")
@RestController
@Log
@RequiredArgsConstructor
public class ResultsController {

    private final ResultsService resultsService;
    private final ResultsMapper resultsMapper;
    private final SimpMessagingTemplate messagingTemplate;

    @GetMapping("/who-knows-better/results")
    public ResponseEntity<List<GetAllResults>> getWhoKnowsBetterQuizResults(
            @PathVariable("weddingId") UUID weddingId,
            @PathVariable("guestId") UUID guestId) {

        List<WhoKnowsBetterQuiz> results = resultsService.findAllResults(weddingId).orElse(List.of());

        List<GetAllResults> resultsDto = resultsMapper.entityToRepresentation(results);

        log.info("📡 Wysyłanie wyników przez WebSocket: " + resultsDto);

        messagingTemplate.convertAndSend("/topic/who-knows-better/results", resultsDto);

        return ResponseEntity.ok(resultsDto);
    }
}
