package org.example.wedding_app_backend.api.repository;

import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.repository.WeddingImageRepository;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.jdbc.EmbeddedDatabaseConnection;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.data.domain.Sort;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@DataJpaTest
@AutoConfigureTestDatabase(connection = EmbeddedDatabaseConnection.H2)
public class WeddingImageRepositoryTests {

    @Autowired
    private WeddingRepository weddingRepository;

    @Autowired
    private WeddingImageRepository weddingImageRepository;

    @Test
    public void findAllByWedding_ReturnsSortedWeddingImages() {

        WeddingEntity weddingEntity = WeddingEntity.builder()
                .id(UUID.randomUUID())
                .name("test name")
                .build();

        List<WeddingImageEntity> weddingImagesList = new ArrayList<>();

        weddingImagesList.add(WeddingImageEntity.builder().id(UUID.randomUUID()).wedding(weddingEntity).likes(1).date(LocalDateTime.now()).build());
        weddingImagesList.add(WeddingImageEntity.builder().id(UUID.randomUUID()).wedding(weddingEntity).likes(3).date(LocalDateTime.now().plusDays(3)).build());
        weddingImagesList.add(WeddingImageEntity.builder().id(UUID.randomUUID()).wedding(weddingEntity).likes(2).date(LocalDateTime.now().plusDays(2)).build());

        weddingRepository.save(weddingEntity);
        weddingImageRepository.saveAll(weddingImagesList);

        List<WeddingImageEntity> weddingImagesListReturn = weddingImageRepository.findAllByWedding(weddingEntity, Sort.by(Sort.Order.desc("likes")));

        Assertions.assertNotNull(weddingImagesListReturn);
        Assertions.assertEquals(3, weddingImagesListReturn.size());
        Assertions.assertEquals(weddingImagesList.get(0), weddingImagesListReturn.get(2));
        Assertions.assertEquals(weddingImagesList.get(1), weddingImagesListReturn.get(0));
        Assertions.assertEquals(weddingImagesList.get(2), weddingImagesListReturn.get(1));
    }

}

