package org.example.wedding_app_backend.api.repository;

import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.repository.WeddingImageRepository;
import org.example.wedding_app_backend.weddingPresent.entity.WeddingPresentEntity;
import org.example.wedding_app_backend.weddingPresent.repository.WeddingPresentRepository;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.jdbc.EmbeddedDatabaseConnection;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.data.domain.Sort;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

@DataJpaTest
@AutoConfigureTestDatabase(connection = EmbeddedDatabaseConnection.H2)
public class WeddingPresentRepositoryTests {

    @Autowired
    private WeddingRepository weddingRepository;

    @Autowired
    private WeddingPresentRepository weddingPresentRepository;

    @Test
    public void findAllByWeddingId_ReturnsWeddingPresents() {

        WeddingEntity weddingEntity = WeddingEntity.builder()
                .id(UUID.randomUUID())
                .name("test wedding")
                .build();

        List<WeddingPresentEntity> weddingPresentsList = new ArrayList<>();

        weddingPresentsList.add(WeddingPresentEntity.builder()
                .id(UUID.randomUUID())
                .wedding(weddingEntity)
                .name("Wedding Gift 1")
                .build());

        weddingPresentsList.add(WeddingPresentEntity.builder()
                .id(UUID.randomUUID())
                .wedding(weddingEntity)
                .name("Wedding Gift 2")
                .build());

        weddingRepository.save(weddingEntity);
        weddingPresentRepository.saveAll(weddingPresentsList);

        Optional<List<WeddingPresentEntity>> result = weddingPresentRepository.findAllByWeddingId(weddingEntity.getId());

        Assertions.assertTrue(result.isPresent());
        Assertions.assertEquals(2, result.get().size());
        Assertions.assertEquals(weddingPresentsList.get(0), result.get().get(0));
        Assertions.assertEquals(weddingPresentsList.get(1), result.get().get(1));
    }

    @Test
    public void findByWeddingIdAndId_ReturnsWeddingPresent() {

        WeddingEntity weddingEntity = WeddingEntity.builder()
                .id(UUID.randomUUID())
                .name("test wedding")
                .build();

        WeddingPresentEntity weddingPresentEntity = WeddingPresentEntity.builder()
                .id(UUID.randomUUID())
                .wedding(weddingEntity)
                .name("Wedding Gift 1")
                .build();

        weddingRepository.save(weddingEntity);
        weddingPresentRepository.save(weddingPresentEntity);

        Optional<WeddingPresentEntity> result = weddingPresentRepository.findByWeddingIdAndId(weddingEntity.getId(), weddingPresentEntity.getId());

        Assertions.assertTrue(result.isPresent());
        Assertions.assertEquals(weddingPresentEntity, result.get());
    }

    @Test
    public void findById_ReturnsWeddingPresent() {

        WeddingEntity weddingEntity = WeddingEntity.builder()
                .id(UUID.randomUUID())
                .name("test wedding")
                .build();

        WeddingPresentEntity weddingPresentEntity = WeddingPresentEntity.builder()
                .id(UUID.randomUUID())
                .wedding(weddingEntity)
                .name("Wedding Gift 1")
                .build();

        weddingRepository.save(weddingEntity);
        weddingPresentRepository.save(weddingPresentEntity);

        Optional<WeddingPresentEntity> result = weddingPresentRepository.findById(weddingPresentEntity.getId());

        Assertions.assertTrue(result.isPresent());
        Assertions.assertEquals(weddingPresentEntity, result.get());
    }

}
