package org.example.wedding_app_backend.api.repository;

import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.model.WhoKnowsBetterQuiz;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.repository.WhoKnowsBetterQuizRepository;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Assertions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.jdbc.EmbeddedDatabaseConnection;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;

import java.util.Optional;
import java.util.UUID;

@DataJpaTest
@AutoConfigureTestDatabase(connection = EmbeddedDatabaseConnection.H2)
public class WeddingWhoKnowsBetterQuizRepositoryTests {

    @Autowired
    private WeddingRepository weddingRepository;

    @Autowired
    private WhoKnowsBetterQuizRepository whoKnowsBetterQuizRepository;

    @Autowired
    private WeddingGuestRepository weddingGuestRepository;


    //tests

    @Test
    public void findByWeddingIdAndWhoKnowsBetterQuestionIndex_ReturnsWeddingWhoKnowsBetterQuestion(){

        WeddingEntity weddingEntity = WeddingEntity.builder()
                .id(UUID.randomUUID())
                .name("test name")
                .build();
        weddingRepository.save(weddingEntity);

        WhoKnowsBetterQuiz quizEntity = WhoKnowsBetterQuiz.builder()
                .id(UUID.randomUUID())
                .wedding(weddingEntity)
                .build();
        whoKnowsBetterQuizRepository.save(quizEntity);

        WeddingGuestEntity guestEntity = WeddingGuestEntity.builder()
                .id(UUID.randomUUID())
                .whoKnowsQuestionIndex(0)
                .wedding(weddingEntity)
                .build();
        weddingGuestRepository.save(guestEntity);


        Optional<WhoKnowsBetterQuiz> result = whoKnowsBetterQuizRepository.findByWeddingIdAndQuestionIndex(weddingEntity.getId(), guestEntity.getWhoKnowsQuestionIndex());

        Assertions.assertNotNull(result);
        Assertions.assertTrue(result.isPresent());
    }

    @Test
    public void updateGroomScore_ShouldIncrementGroomScore(){

        WeddingEntity weddingEntity = WeddingEntity.builder()
                .id(UUID.randomUUID())
                .name("test name")
                .build();
        weddingRepository.save(weddingEntity);

        WhoKnowsBetterQuiz betterQuiz = WhoKnowsBetterQuiz.builder()
                .id(UUID.randomUUID())
                .wedding(weddingEntity)
                .groomScore(0)
                .brideScore(1)
                .build();
        whoKnowsBetterQuizRepository.save(betterQuiz);

        whoKnowsBetterQuizRepository.updateGroomScore(weddingEntity.getId(), betterQuiz.getId());

        WhoKnowsBetterQuiz updatedQuiz = whoKnowsBetterQuizRepository.findById(betterQuiz.getId()).get();
        Assertions.assertNotNull(updatedQuiz);
        Assertions.assertEquals(1, updatedQuiz.getGroomScore());
        Assertions.assertEquals(1, updatedQuiz.getBrideScore());
    }

    @Test
    public void updateBrideScore_ShouldIncrementBrideScore(){

        WeddingEntity weddingEntity = WeddingEntity.builder()
                .id(UUID.randomUUID())
                .name("test name")
                .build();
        weddingRepository.save(weddingEntity);

        WhoKnowsBetterQuiz betterQuiz = WhoKnowsBetterQuiz.builder()
                .id(UUID.randomUUID())
                .wedding(weddingEntity)
                .groomScore(1)
                .brideScore(0)
                .build();
        whoKnowsBetterQuizRepository.save(betterQuiz);

        whoKnowsBetterQuizRepository.updateBrideScore(weddingEntity.getId(), betterQuiz.getId());

        WhoKnowsBetterQuiz updatedQuiz = whoKnowsBetterQuizRepository.findById(betterQuiz.getId()).get();
        Assertions.assertNotNull(updatedQuiz);
        Assertions.assertEquals(1, updatedQuiz.getGroomScore());
        Assertions.assertEquals(1, updatedQuiz.getBrideScore());
    }

}
