package org.example.wedding_app_backend.api.service;

import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.repository.WeddingImageRepository;
import org.example.wedding_app_backend.weddingImage.service.WeddingImageService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
public class WeddingImageServiceTests {

    @Mock
    private WeddingImageRepository weddingImageRepository;

    @Mock
    private WeddingGuestRepository weddingGuestRepository;

    @InjectMocks
    private WeddingImageService weddingImageService;

    private WeddingEntity weddingEntity;

    private WeddingImageEntity weddingImageEntity;

    private WeddingGuestEntity weddingGuestEntity;

    @BeforeEach
    public void setUp() {
        weddingEntity = WeddingEntity.builder().id(UUID.randomUUID()).build();
        weddingImageEntity = WeddingImageEntity.builder().id(UUID.randomUUID()).wedding(weddingEntity).build();
        weddingGuestEntity = WeddingGuestEntity.builder().id(UUID.randomUUID()).wedding(weddingEntity).build();

        weddingEntity.setImages(new ArrayList<WeddingImageEntity>(List.of(weddingImageEntity)));
        weddingImageEntity.setLikedByGuest(new ArrayList<>());
        weddingGuestEntity.setLikedImages(new ArrayList<>());
    }

    @Test
    public void updateLikes_ShouldIncreaseImageLikes_WhenImageIsNotLiked() {
        int initialLikes = 0;

        weddingImageEntity.setLikes(initialLikes);

        when(weddingGuestRepository.findById(weddingGuestEntity.getId())).thenReturn(Optional.of(weddingGuestEntity));
        when(weddingImageRepository.findById(weddingImageEntity.getId())).thenReturn(Optional.of(weddingImageEntity));

        when(weddingGuestRepository.save(weddingGuestEntity)).thenReturn(weddingGuestEntity);
        when(weddingImageRepository.save(weddingImageEntity)).thenReturn(weddingImageEntity);
        
        weddingImageService.updateLikes(weddingGuestEntity.getId(), weddingImageEntity.getId(), true);

        Assertions.assertEquals(initialLikes + 1, weddingImageEntity.getLikes());
    }

    @Test
    public void UpdateLikes_ShouldNotIncreaseImageLikes_WhenImageIsLiked() {
        int initialLikes = 1;

        weddingImageEntity.setLikes(initialLikes);

        weddingGuestEntity.setLikedImages(new ArrayList<>(List.of(weddingImageEntity)));

        when(weddingGuestRepository.findById(weddingGuestEntity.getId())).thenReturn(Optional.of(weddingGuestEntity));
        when(weddingImageRepository.findById(weddingImageEntity.getId())).thenReturn(Optional.of(weddingImageEntity));

        when(weddingGuestRepository.save(weddingGuestEntity)).thenReturn(weddingGuestEntity);
        when(weddingImageRepository.save(weddingImageEntity)).thenReturn(weddingImageEntity);

        weddingImageService.updateLikes(weddingGuestEntity.getId(), weddingImageEntity.getId(), true);

        Assertions.assertEquals(initialLikes, weddingImageEntity.getLikes());
    }
}
