package org.example.wedding_app_backend.api.service;

import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.weddingPresent.dto.GetWeddingPresentResponse;
import org.example.wedding_app_backend.weddingPresent.entity.WeddingPresentEntity;
import org.example.wedding_app_backend.weddingPresent.repository.WeddingPresentRepository;
import org.example.wedding_app_backend.weddingPresent.service.WeddingPresentService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.aot.generate.AccessControl;

import java.io.IOException;

import java.util.UUID;

import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
public class WeddingPresentServiceTests {

    @Mock
    private WeddingPresentRepository weddingPresentRepository;

    @Mock
    private WeddingGuestRepository weddingGuestRepository;

    @InjectMocks
    private WeddingPresentService weddingPresentService;

    private WeddingEntity weddingEntity;

    private WeddingPresentEntity weddingPresentEntity;

    @BeforeEach
    public void setUp() {
        weddingEntity = WeddingEntity.builder().id(UUID.randomUUID()).build();
        weddingPresentEntity = WeddingPresentEntity.builder()
                .id(UUID.randomUUID())
                .name("Gift Name")
                .description("Gift Description")
                .linkToPresent("http://example.com")
                .wedding(weddingEntity)
                .build();

    }

    @Test
    void updatePresentReservedBy_ShouldUpdateReservedBy() throws IOException {
        GetWeddingPresentResponse response = new GetWeddingPresentResponse();
        response.setReservedBy((UUID.randomUUID()));

        weddingPresentService.updatePresentReservedBy(weddingPresentEntity, response);

        Assertions.assertEquals(response.getReservedBy(), weddingPresentEntity.getReservedBy());
    }

    @Test
    void updatePresentParametersModifiableByUser_ShouldUpdateFields() throws IOException {
        GetWeddingPresentResponse request = new GetWeddingPresentResponse();
        request.setName("New Name");
        request.setDescription("New Description");
        request.setLinkToPresent("http://new-link.com");

        weddingPresentService.updatePresentParametersModifiableByUser(weddingPresentEntity, request);

        Assertions.assertEquals(request.getName(), weddingPresentEntity.getName());
        Assertions.assertEquals(request.getDescription(), weddingPresentEntity.getDescription());
        Assertions.assertEquals(request.getLinkToPresent(), weddingPresentEntity.getLinkToPresent());
    }

    @Test
    void createWeddingPresent_ShouldCreateAndSavePresent() throws IOException {
        GetWeddingPresentResponse request = new GetWeddingPresentResponse();
        request.setName("Gift 1");
        request.setDescription("A wonderful gift");
        request.setLinkToPresent("http://gift.com");

        when(weddingPresentRepository.save(any(WeddingPresentEntity.class))).thenAnswer(invocation -> {
            WeddingPresentEntity savedPresent = invocation.getArgument(0);
            savedPresent.setId(UUID.randomUUID());
            return savedPresent;
        });

        weddingPresentService.createWeddingPresent(request, weddingEntity);

        verify(weddingPresentRepository, times(1)).save(argThat(present ->
                present.getName().equals(request.getName()) &&
                        present.getDescription().equals(request.getDescription()) &&
                        present.getLinkToPresent().equals(request.getLinkToPresent()) &&
                        present.getWedding().equals(weddingEntity) &&
                        present.getId() != null
        ));
    }

    @Test
    void deletePresent_ShouldDeletePresent() throws IOException {
        doNothing().when(weddingPresentRepository).delete(weddingPresentEntity);

        weddingPresentService.deletePresent(weddingPresentEntity);

        verify(weddingPresentRepository, times(1)).delete(weddingPresentEntity);
    }

}
