package org.example.wedding_app_backend.api.service;

import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.weddingQuiz.entity.WeddingQuizEntity;
import org.example.wedding_app_backend.weddingQuiz.repository.WeddingQuizRepository;
import org.example.wedding_app_backend.weddingQuiz.service.WeddingQuizService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
public class WeddingQuizServiceTests {

    @Mock
    private WeddingQuizRepository weddingQuizRepository;

    @Mock
    private WeddingGuestRepository weddingGuestRepository;

    @InjectMocks
    private WeddingQuizService weddingQuizService;

    private WeddingEntity weddingEntity;

    private WeddingQuizEntity weddingQuizEntity;

    private WeddingGuestEntity weddingGuestEntity;

    @BeforeEach
    public void setUp() {
        weddingEntity = WeddingEntity.builder().id(UUID.randomUUID()).build();
        weddingGuestEntity = WeddingGuestEntity.builder().id(UUID.randomUUID()).wedding(weddingEntity).build();
        weddingQuizEntity = WeddingQuizEntity.builder().id(UUID.randomUUID()).wedding(weddingEntity).build();

    }

    @Test
    public void findQuestionByGuestId_ShouldReturnNextQuestion_WhenIndexIsLowerThanNumberOfQuestions() {
        when(weddingQuizRepository.findQuestionById(weddingGuestEntity.getQuestionIndex(), weddingEntity.getId(), weddingGuestEntity.getQuizSeed()))
                .thenReturn(Optional.of(weddingQuizEntity));

        Optional<WeddingQuizEntity> result=weddingQuizService.findQuestionByGuestId(weddingGuestEntity, weddingEntity.getId());

        Assertions.assertTrue(result.isPresent());
        Assertions.assertEquals(weddingQuizEntity, result.get());
    }

    @Test
    public void findQuestionByGuestId_ShouldReturnEmpty_WhenIndexIsHigherThanNumberOfQuestions() {
        when(weddingQuizRepository.findQuestionById(weddingGuestEntity.getQuestionIndex(), weddingEntity.getId(), weddingGuestEntity.getQuizSeed()))
                .thenReturn(Optional.empty());

        Optional<WeddingQuizEntity> result=weddingQuizService.findQuestionByGuestId(weddingGuestEntity, weddingEntity.getId());

        Assertions.assertFalse(result.isPresent());
    }

}
