package org.example.wedding_app_backend.api.service;

import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.model.WhoKnowsBetterQuiz;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.repository.WhoKnowsBetterQuizRepository;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.service.WhoKnowsBetterQuizService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.Optional;
import java.util.UUID;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
public class WeddingWhoKnowsBetterQuizServiceTests {

    @Mock
    private WhoKnowsBetterQuizRepository whoKnowsBetterQuizRepository;

    @Mock
    private WeddingRepository weddingRepository;

    @Mock
    private WeddingGuestRepository weddingGuestRepository;

    @InjectMocks
    private WhoKnowsBetterQuizService whoKnowsBetterQuizService;

    private WeddingEntity weddingEntity;
    private WeddingGuestEntity weddingGuestEntity;
    private WhoKnowsBetterQuiz quizEntity;
    private UUID weddingId;
    private UUID guestId;
    private UUID quizId;

    @BeforeEach
    public void setUp() {
        weddingId = UUID.randomUUID();
        guestId = UUID.randomUUID();
        quizId = UUID.randomUUID();

        weddingEntity = WeddingEntity.builder()
                .id(weddingId)
                .name("test wedding")
                .build();

        weddingGuestEntity = WeddingGuestEntity.builder()
                .id(guestId)
                .wedding(weddingEntity)
                .whoKnowsQuestionIndex(1)
                .build();

        quizEntity = WhoKnowsBetterQuiz.builder()
                .id(quizId)
                .wedding(weddingEntity)
                .question("Who is more likely to be late?")
                .groomScore(3)
                .brideScore(2)
                .build();
    }

    @Test
    public void findQuestionByGuestId_ShouldReturnNextQuestion_WhenIndexIsLowerThanNumberOfQuestions() {

        when(whoKnowsBetterQuizRepository.findByWeddingIdAndQuestionIndex(weddingId, weddingGuestEntity.getWhoKnowsQuestionIndex()))
                .thenReturn(Optional.of(quizEntity));

        Optional<WhoKnowsBetterQuiz> result = whoKnowsBetterQuizService.findQuestionByGuestId(weddingGuestEntity, weddingId);

        Assertions.assertTrue(result.isPresent());
        Assertions.assertEquals(quizEntity, result.get());
    }

    @Test
    public void findQuestionByGuestId_ShouldReturnEmpty_WhenIndexIsHigherThanNumberOfQuestions() {

        when(whoKnowsBetterQuizRepository.findByWeddingIdAndQuestionIndex(weddingId, weddingGuestEntity.getWhoKnowsQuestionIndex()))
                .thenReturn(Optional.empty());

        Optional<WhoKnowsBetterQuiz> result = whoKnowsBetterQuizService.findQuestionByGuestId(weddingGuestEntity, weddingId);

        Assertions.assertFalse(result.isPresent());
    }

    @Test
    public void updateQuestionCounter_ShouldUpdateGroomScoreAndReturnUpdatedQuiz() {

        when(weddingGuestRepository.findById(guestId))
                .thenReturn(Optional.of(weddingGuestEntity));

        when(whoKnowsBetterQuizRepository.findByWeddingIdAndQuestionIndex(weddingId, weddingGuestEntity.getWhoKnowsQuestionIndex()))
                .thenReturn(Optional.of(quizEntity));

        WhoKnowsBetterQuiz updatedQuiz = WhoKnowsBetterQuiz.builder()
                .id(quizId)
                .wedding(weddingEntity)
                .question("Who is more likely to be late?")
                .groomScore(4)
                .brideScore(2)
                .build();
        when(whoKnowsBetterQuizRepository.findById(quizId))
                .thenReturn(Optional.of(updatedQuiz));

        WhoKnowsBetterQuiz result = whoKnowsBetterQuizService.updateQuestionCounter(weddingId, guestId, "groom");

        verify(whoKnowsBetterQuizRepository).updateGroomScore(weddingId, quizId);
        verify(whoKnowsBetterQuizRepository, never()).updateBrideScore(any(), any());

        Assertions.assertEquals(updatedQuiz, result);
        Assertions.assertEquals(4, result.getGroomScore());
        Assertions.assertEquals(2, result.getBrideScore());
    }

    @Test
    public void updateQuestionCounter_ShouldUpdateBrideScoreAndReturnUpdatedQuiz() {

        when(weddingGuestRepository.findById(guestId))
                .thenReturn(Optional.of(weddingGuestEntity));

        when(whoKnowsBetterQuizRepository.findByWeddingIdAndQuestionIndex(weddingId, weddingGuestEntity.getWhoKnowsQuestionIndex()))
                .thenReturn(Optional.of(quizEntity));

        WhoKnowsBetterQuiz updatedQuiz = WhoKnowsBetterQuiz.builder()
                .id(quizId)
                .wedding(weddingEntity)
                .question("Who is more likely to be late?")
                .groomScore(3)
                .brideScore(3)
                .build();
        when(whoKnowsBetterQuizRepository.findById(quizId))
                .thenReturn(Optional.of(updatedQuiz));

        WhoKnowsBetterQuiz result = whoKnowsBetterQuizService.updateQuestionCounter(weddingId, guestId, "bride");

        verify(whoKnowsBetterQuizRepository).updateBrideScore(weddingId, quizId);
        verify(whoKnowsBetterQuizRepository, never()).updateGroomScore(any(), any());

        Assertions.assertEquals(updatedQuiz, result);
        Assertions.assertEquals(3, result.getGroomScore());
        Assertions.assertEquals(3, result.getBrideScore());
    }
}