import {Component, Input, OnInit} from '@angular/core';
import {MessageService} from 'primeng/api';
import {GuestType} from '../../models/guests.type';
import {ConfirmationGroupService} from '../../services/confirmation-group.service';
import {DataViewModule} from 'primeng/dataview';
import {CommonModule, NgClass} from '@angular/common';
import {TagModule} from 'primeng/tag';
import {Button} from 'primeng/button';
import {GuestsService} from '../../services/guests.service';
import {ToastModule} from 'primeng/toast';

@Component({
  selector: 'app-confirmation',
  standalone: true,
  imports: [
    DataViewModule,
    NgClass,
    CommonModule,
    TagModule,
    Button,
    ToastModule
  ],
  templateUrl: './confirmation.component.html',
  styleUrl: './confirmation.component.css'
})
export class ConfirmationComponent implements OnInit {

  confirmationGuests: GuestType[] = [];

  @Input()
  weddingId: string = '';

  @Input()
  guestId: string = '';

  constructor(
    private confirmationGroupService: ConfirmationGroupService,
    private guestService: GuestsService,
    private messageService: MessageService
  ) {
  }

  ngOnInit() {
    this.confirmationGroupService.getConfirmationGroupGuests(this.weddingId, this.guestId).subscribe({
      next: guests => {this.confirmationGuests = guests;},
      error: err => {console.log(err);}
    })
  }

  confirmButtonAction(guest: GuestType, presence: boolean) {
    this.guestService.patchCharacterPresenceConfirmation(this.weddingId, this.guestId, presence).subscribe({
      next: n => {
        switch (presence) {
          case true:
            this.messageService.add({ severity: 'success', summary: 'Potwierdzono obecność', detail: guest.guestName + ' ' , key: 's', life: 3000 });
            break;
          case false:
            this.messageService.add({ severity: 'warning', summary: 'Potwierdzono nieobecność', detail: guest.guestName + ' ', key: 's', life: 3000 });
            break;
        }
        guest.presenceConfirmed = presence;
      }
    })
  }

  getTagSeverity(guest: GuestType) {
    switch (guest.presenceConfirmed) {
      case null:
        return 'warning';
      case true:
        return 'success';
      case false:
        return 'danger';
    }
  }

  getTagValue(guest: GuestType) {
    switch (guest.presenceConfirmed) {
      case null:
        return 'Brak potwierdzenia';
      case true:
        return 'Obecność potwierdzona';
      case false:
        return 'Nieobecność potwierdzona';
    }
  }
}
