import { ComponentFixture, TestBed } from '@angular/core/testing';

import { NavComponent } from './nav.component';
import {SharedService} from '../../services/shared.service';
import {provideRouter} from '@angular/router';
import {provideHttpClient} from '@angular/common/http';
import {provideHttpClientTesting} from '@angular/common/http/testing';
import {GalleryService} from '../../services/gallery.service';
import {BehaviorSubject} from 'rxjs';

describe('NavComponent', () => {
  let component: NavComponent;
  let fixture: ComponentFixture<NavComponent>;
  let native: HTMLElement;

  const weddingId = '4d35e8f0-b155-4e79-b5e8-f0b1550e7940';
  const guestId = 'e501bf41-d3d2-417d-81bf-41d3d2217d0a';

  let sharedService: jasmine.SpyObj<SharedService>;

  beforeEach(async () => {
    const spy = jasmine.createSpyObj<SharedService>('SharedService', ['currentMessage']);

    await TestBed.configureTestingModule({
      imports: [NavComponent],
      providers: [
        provideRouter([]),
        provideHttpClient(),
        provideHttpClientTesting(),
        {
          provide: SharedService,
          useValue: spy
        }
      ]
    })
    .compileComponents();

    fixture = TestBed.createComponent(NavComponent);
    native = fixture.nativeElement;
    component = fixture.componentInstance;
    component.weddingId = weddingId;
    component.guestId = guestId;
    component.items = [
        { icon: 'assets/icons/home.svg', label: 'Home', command: () => {}},
        { icon: 'assets/icons/picture.svg', label: 'Gallery', command: () => {}},
        { icon: 'assets/icons/games.svg', label: 'Games', command: () => {}}
      ];
    sharedService = TestBed.inject(SharedService) as jasmine.SpyObj<SharedService>;
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should show confirmation navigation button if message is true', () => {
    sharedService.currentMessage = new BehaviorSubject(true);
    fixture.detectChanges();

    const navElement = native.querySelector('nav');
    expect(navElement).toBeTruthy();
    const buttonElement = native.querySelector('.p-button');
    expect(buttonElement).toBeNull();

    const labels = Array.from(navElement!.innerText.split('\n'));
    expect(labels).toContain('Home');
    expect(labels).toContain('Gallery');
    expect(labels).toContain('Games');
  });

  it('should show wedding attractions navigation button if message is false', () => {
    sharedService.currentMessage = new BehaviorSubject(false);
    fixture.detectChanges();

    const buttonElement = native.querySelector('.p-button');
    expect(buttonElement).toBeTruthy();
    const navElement = native.querySelector('nav');
    expect(navElement).toBeNull();
  });
});
