import {Component, Input, OnInit} from '@angular/core';
import {NavigationType} from '../../models/navigation.type';
import {TabMenuModule} from 'primeng/tabmenu';
import {ActivatedRoute, NavigationEnd, Router} from '@angular/router';
import {WeddingType} from '../../models/wedding.type';
import {WeddingService} from '../../services/wedding.service';
import {SharedService} from '../../services/shared.service';
import {NgIf} from '@angular/common';
import {Button} from 'primeng/button';

@Component({
  selector: 'app-nav',
  standalone: true,
  templateUrl: './nav.component.html',
  imports: [
    TabMenuModule,
    NgIf,
    Button
  ],
  styleUrl: './nav.component.css',
host: { ngSkipHydration: 'true' }
})
export class NavComponent implements OnInit {

  isWeddingDay: boolean = true;

  weddingId!: string;

  guestId!: string;

  items: Array<NavigationType> = [
    { icon: 'assets/icons/home.svg', label: 'Home', command: () => this.navigateTo()},
    { icon: 'assets/icons/picture.svg', label: 'Gallery', command: () => this.navigateTo('gallery')},
    { icon: 'assets/icons/games.svg', label: 'Games', command: () => this.navigateTo('games')}
  ];

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private sharedService: SharedService,) {}

  ngOnInit(): void {
    this.weddingId = this.route.firstChild?.snapshot.params['weddingId'];
    this.guestId = this.route.firstChild?.snapshot.params['guestId'];

    this.sharedService.currentMessage.subscribe(message => {
      this.isWeddingDay = message;
    })
  }

  protected navigateTo(...paths: string[]) {
    this.router.navigate(['/wedding', this.weddingId, this.guestId, ...paths]);
  }
}
