import {Component, CUSTOM_ELEMENTS_SCHEMA, OnInit} from '@angular/core';
import {DataViewModule} from 'primeng/dataview';
import {TagModule} from 'primeng/tag';
import {Button} from 'primeng/button';
import {NgClass, NgForOf, NgIf} from '@angular/common';
import {PresentType} from '../../models/presents.type';
import {GuestType} from '../../models/guests.type';
import {ActivatedRoute, Router} from '@angular/router';
import {PresentService} from '../../services/present.service';
import {ConfirmationService, MessageService} from 'primeng/api';
import {DialogModule} from 'primeng/dialog';
import {AutoCompleteModule} from 'primeng/autocomplete';
import {DropdownModule} from 'primeng/dropdown';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';
import {InputTextareaModule} from 'primeng/inputtextarea';
import {RadioButtonModule} from 'primeng/radiobutton';
import {catchError} from 'rxjs';
import {KeycloakService} from '../../keycloak/keycloak.service';
import {ConfirmDialogModule} from 'primeng/confirmdialog';
import {ToastModule} from 'primeng/toast';

@Component({
  selector: 'app-present-user',
  standalone: true,
  imports: [
    DataViewModule,
    TagModule,
    Button,
    NgClass,
    NgForOf,
    DialogModule,
    AutoCompleteModule,
    DropdownModule,
    NgIf,
    ReactiveFormsModule,
    FormsModule,
    InputTextareaModule,
    RadioButtonModule,
    ConfirmDialogModule,
    ToastModule
  ],
  providers: [MessageService, ConfirmationService],
  templateUrl: './present-user.component.html',
  styleUrl: './present-user.component.css',
  schemas: [CUSTOM_ELEMENTS_SCHEMA],
  host: { ngSkipHydration: 'true' }
})
export class PresentUserComponent implements OnInit {

  weddingPresents: PresentType[] =[];

  presentForm: PresentType={
    id:'',
    name:'',
    linkToPresent:'',
    description:'',
    reservedBy: ''
  };

  presentDialog: boolean=false;
  submitted:boolean=false;

  constructor(private PresentService: PresentService,
              private confirmationService: ConfirmationService,
              private messageService: MessageService,
              private keycloakService: KeycloakService,
  ) {}

  ngOnInit(): void {
    if(this.keycloakService.getToken()){
      this.getPresents();
    }
  }

  getPresents(){
    this.PresentService.getPresentsByWeddingId().pipe(
      catchError((error) => {
        console.error('Error fetching quiz state:', error);
        throw error;
      })
    ).subscribe((weddingPresents) => {
      this.weddingPresents = weddingPresents;
    });
  }

  addPresent() {
    this.presentForm = {id: '',
      name: '',
      linkToPresent: '',
      description: '',
      reservedBy: ''};
    this.submitted = false;
    this.presentDialog = true;
  }

  editPresent(present: PresentType) {
    this.presentForm = { ...present };
    this.presentDialog = true;
  }

  deletePresent(present: PresentType) {
    this.confirmationService.confirm({
      message: 'Czy na pewno chcesz usunąć ten prezent?',
      header: 'Potwierdzenie',
      acceptLabel: 'Tak, usuń',
      rejectLabel: 'Anuluj',
      acceptIcon: 'pi pi-check mr-2',
      rejectIcon: 'pi pi-times mr-2',
      rejectButtonStyleClass: 'p-button-sm',
      acceptButtonStyleClass: 'p-button-outlined p-button-sm',
      accept: () => {
        this.PresentService.deletePresentByUser(present).subscribe({
          next: () => {
            this.getPresents();
            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Prezent usunięty',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd!',
              detail: 'Nie udało się usunąć prezentu',
              life: 3000
            });
          }
        });
      }
    });
  }

  hideDialog() {
    this.presentDialog = false;
    this.submitted = false;
  }

  savePresent() {
    this.submitted = true;

    if (this.presentForm.name?.trim()) {
      if (this.presentForm.id) {
        this.PresentService.patchPresentDataModifiableByUser(this.presentForm).subscribe({
          next: () => {
            this.getPresents();
            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Dane zminenione',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd',
              detail: 'Nie udało się zmienić danych',
              life: 3000
            });
          }
        });
      } else {
        this.PresentService.postPresentToCreate(this.presentForm).subscribe({
          next: () => {
            this.getPresents();
            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Prezent dodany',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd',
              detail: 'Nie udało się utworzyć prezentu',
              life: 3000
            });
          }
        });
      }

      this.weddingPresents = [...this.weddingPresents];
      this.presentDialog = false;
      this.presentForm = { id:'',
        name:'',
        linkToPresent:'',
        description: '',
        reservedBy: '' };
    }
  }

}
