import {Component, OnInit} from '@angular/core';
import {GuestsService} from '../../../services/guests.service';
import {GuestType} from '../../../models/guests.type';
import {ChartModule} from 'primeng/chart';
import ChartDataLabels from 'chartjs-plugin-datalabels';
import { Chart } from 'chart.js';


@Component({
  selector: 'app-dashboard-tile-guests',
  standalone: true,
  imports: [
    ChartModule
  ],
  templateUrl: './dashboard-tile-guests.component.html',
  styleUrl: './dashboard-tile-guests.component.css'
})
export class DashboardTileGuestsComponent implements OnInit {

  guests!: GuestType[];

  presence: number[] = [0, 0, 0];

  data: any;

  options: any;

  constructor(
    private guestsService: GuestsService
  ) {
    Chart.register(ChartDataLabels);
  }

  ngOnInit() {
    this.guestsService.getGuestsByWeddingId().subscribe({
      next: guests=> {
        this.guests = guests;
        this.calculatePresence()
        this.initializeChart();
      },
      error: error => console.log(error)
    })
  }

  calculatePresence() {
    this.presence = [0, 0, 0];

    for (const guest of this.guests) {
      switch (guest.presenceConfirmed) {
        case true:
          this.presence[0] += 1; // OBECNI
          break;
        case false:
          this.presence[1] += 1; // NIEOBECNI
          break;
        case null:
          this.presence[2] += 1; // BRAK POTWIERDZENIA
          break;
      }
    }
  }

  initializeChart() {
    const documentStyle = getComputedStyle(document.documentElement);
    const textColor = documentStyle.getPropertyValue('--text-color');

    this.data = {
      labels: ['OBECNI', 'NIEOBECNI', 'BRAK POTWIERDZENIA'],
      datasets: [
        {
          data: this.presence,
          backgroundColor: [documentStyle.getPropertyValue('--green-500'), documentStyle.getPropertyValue('--red-500'), documentStyle.getPropertyValue('--yellow-500')],
          hoverBackgroundColor: [documentStyle.getPropertyValue('--green-400'), documentStyle.getPropertyValue('--red-400'), documentStyle.getPropertyValue('--yellow-400')]
        }
      ]
    };

    this.options = {
      cutout: '60%',
      plugins: {
        legend: {
          labels: {
            usePointStyle: true,
            color: '#fff',
            font: {
              size: 14,
              weight: 'bold'
            }
          }
        },
        datalabels: {
          color: '#fff',
          font: {
            size: 16,
            weight: 'bold'
          },
          formatter: (value: number) => {
            return value > 0 ? value.toString() : '';
          }
        }
      }
    };
  }
}
