import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { WhoKnowsBetterResultsService } from '../../services/who-knows-better-results.service';
import { CommonModule } from '@angular/common';
import { WhoKnowsBetterResultsType } from '../../models/whoKnowsBetterResults.type';
import { filter } from 'rxjs/operators';
import { CardModule } from 'primeng/card';
import { ProgressBarModule } from 'primeng/progressbar';
import { ToastModule } from 'primeng/toast';

@Component({
  selector: 'app-who-knows-better-summary',
  standalone: true,
  imports: [CommonModule, CardModule, ProgressBarModule, ToastModule],
  templateUrl: './who-knows-better-summary.component.html',
  styleUrls: ['./who-knows-better-summary.component.css'],
  encapsulation: ViewEncapsulation.None,
  host: { ngSkipHydration: 'true' }
})
export class WhoKnowsBetterSummaryComponent implements OnInit {
  weddingId: string = '';
  guestId: string = '';
  results: WhoKnowsBetterResultsType[] = [];

  constructor(
    private route: ActivatedRoute,
    private whoKnowsBetterResultsService: WhoKnowsBetterResultsService
  ) {}

  ngOnInit(): void {
    this.weddingId = this.route.snapshot.paramMap.get('weddingId') || '';
    this.guestId = this.route.snapshot.paramMap.get('guestId') || '';
    if (this.weddingId && this.guestId) {
      this.initializeWebSocket();
    } else {
      console.error('Brak weddingId lub guestId!');
    }
  }

  initializeWebSocket(): void {
    this.whoKnowsBetterResultsService.connect(this.weddingId, this.guestId);
    this.whoKnowsBetterResultsService.results$
      .pipe(filter(data => data !== null))
      .subscribe((data) => {
        if (Array.isArray(data)) {
          data.forEach((updatedQuestion) => this.updateResults(updatedQuestion));
        } else {
          this.updateResults(data);
        }
      });
  }

  private updateResults(updatedQuestion: WhoKnowsBetterResultsType): void {
    if (!updatedQuestion || !updatedQuestion.id) {
      return;
    }
    const index = this.results.findIndex(q => q.id === updatedQuestion.id);
    if (index !== -1) {
      this.results[index] = updatedQuestion;
    } else {
      this.results.push(updatedQuestion);
    }
  }
}
