import {ChangeDetectorRef, Pipe, PipeTransform} from '@angular/core';
import {BehaviorSubject, distinctUntilChanged, map, Subscription, switchMap, tap} from 'rxjs';
import {DomSanitizer, SafeUrl} from '@angular/platform-browser';
import {HttpClient} from '@angular/common/http';
import {filter} from 'rxjs/operators';

@Pipe({
  name: 'securedImage',
  standalone: true,
  pure: false
})
export class MockSecureImagePipe implements PipeTransform {

  private subscription = new Subscription();

  private transformValue = new BehaviorSubject<string>('');

  private latestValue!: string | SafeUrl;

  constructor(
    private http: HttpClient,
    private sanitizer: DomSanitizer,
    private cdr: ChangeDetectorRef
  ) {
    this.initSubscription();
  }

  ngOnDestroy(): void {
    this.subscription.unsubscribe();
  }

  transform(url: string): any {
    return 'blob:nodedata:4172ae66-d889-4030-b01f-af305e911392';
  }

  private initSubscription() {
    const transformSubscription = this.transformValue
      .subscribe();
    this.subscription.add(transformSubscription);
  }
}
