import { SecuredImagePipe } from './secured-image.pipe';
import { TestBed } from '@angular/core/testing';
import { HttpTestingController, provideHttpClientTesting } from '@angular/common/http/testing';
import { provideHttpClient } from '@angular/common/http';
import { ChangeDetectorRef } from '@angular/core';

describe(SecuredImagePipe.name, () => {

  let instance: SecuredImagePipe;

  let http: HttpTestingController;

  let cdr: ChangeDetectorRef;

  beforeEach(async () => {
    TestBed.configureTestingModule({
      imports: [],
      providers: [
        provideHttpClient(),
        provideHttpClientTesting(),
        SecuredImagePipe,
        {
          provide: ChangeDetectorRef,
          useValue: {
            markForCheck: jasmine.createSpy('markForCheck')
          }
        }
      ]
    });
    instance = TestBed.inject(SecuredImagePipe);
    http = TestBed.inject(HttpTestingController);
    cdr = TestBed.inject(ChangeDetectorRef);
  })

  it('create an instance', () => {
    expect(instance).toBeTruthy();
  });

  it('should transform a URL into a sanitized blob URL', () => {
    const testUrl = 'https://example.com/image.jpg';
    const mockBlob = new Blob(['mock data'], {type: 'image/jpeg'});

    instance.transform(testUrl);

    const req = http.expectOne(testUrl);
    expect(req.request.method).toBe('GET');

    req.flush(mockBlob);

    setTimeout(() => {
      const actual = instance.transform(testUrl);
      expect(actual).toEqual(jasmine.any(Object));
      expect(cdr.markForCheck).toHaveBeenCalled();
    }, 0);
  });

});

