import {inject, Injectable} from '@angular/core';
import {HttpClient, HttpParams} from '@angular/common/http';
import {Observable} from 'rxjs';
import {ImageType} from '../models/image.type';
import { v4 as uuidv4 } from 'uuid';

@Injectable({
  providedIn: 'root'
})
export class GalleryService {

  constructor(private http:HttpClient) { }

  getGalleryByWeddingId(weddingId: string, guestId: string, sort: string): Observable<[ImageType]> {
    let params = new HttpParams().set('sort', sort);
    return this.http.get<[ImageType]>(`/api/weddings/${weddingId}/guests/${guestId}/images`, {params: params});
  }

  getGalleryImage(weddingId: string, imageId: string): Observable<Blob> {
    return this.http.get(`/api/weddings/${weddingId}/images/${imageId}`, { responseType: 'blob' });
  }

  postImage(file: File, weddingId: string, guestId: string): Observable<any> {
    const formData = new FormData();
    const imageId = uuidv4();

    if (file.size === 0) {
      console.error('File is empty');
    }
    formData.append('image', file, file.name);

    return this.http.post(`/api/weddings/${weddingId}/images/${imageId}`, formData);
  }

  changeImageLikes(weddingId : string, guestId: string, imageId : string, liked: boolean): Observable<any> {
    let params = new HttpParams().set('liked', liked.toString());
    return this.http.patch(`/api/weddings/${weddingId}/guests/${guestId}/images/${imageId}`, {}, {params: params});
  }
}
