
import { Injectable } from '@angular/core';
import { Client, Frame, IMessage } from '@stomp/stompjs';
import { BehaviorSubject } from 'rxjs';
import { KeycloakService } from '../keycloak/keycloak.service';
import { WhoKnowsBetterResultsType } from '../models/whoKnowsBetterResults.type';


@Injectable({
  providedIn: 'root'
})
export class WhoKnowsBetterResultsService {
  private stompClient!: Client;
  public results$ = new BehaviorSubject<WhoKnowsBetterResultsType[]>([]);

  constructor(private keycloakService: KeycloakService) {}

  async connect(weddingId: string, guestId: string): Promise<void> {
    try {

      const serverUrl = 'ws://localhost:8080/ws';
      this.stompClient = new Client({
        webSocketFactory: () => new WebSocket(serverUrl),
        reconnectDelay: 5000,
        debug: (str) => console.log(str)
      });

      this.stompClient.onConnect = (frame: Frame) => {
        console.log('WebSocket connected:', frame);
        this.fetchInitialResults(weddingId, guestId);
        this.stompClient.subscribe('/topic/who-knows-better/results', (message: IMessage) => {
          if (message.body) {
            const updatedQuestion = JSON.parse(message.body);
            console.log('Received update:', updatedQuestion);
            this.updateResults(updatedQuestion);
          }
        });
      };

      this.stompClient.onStompError = (frame: Frame) => {
        console.error('WebSocket error: ' + frame.headers['message']);
        console.error('Additional details: ' + frame.body);
      };

      this.stompClient.activate();
    } catch (error) {
      console.error('Error while connecting to WebSocket:', error);
    }
  }

  private async fetchInitialResults(weddingId: string, guestId: string) {
    try {
      const response = await fetch(`http://localhost:8080/api/wedding/${weddingId}/${guestId}/games/who-knows-better/results`);
      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`);
      }
      const results: WhoKnowsBetterResultsType[] = await response.json();
      console.log('Initial results fetched:', results);
      this.results$.next(results);
    } catch (error) {
      console.error('Error fetching initial results:', error);
    }
  }

  private updateResults(updatedQuestion: WhoKnowsBetterResultsType) {
    const currentResults = this.results$.getValue();
    const index = currentResults.findIndex((q) => q.id === updatedQuestion.id);
    if (index > -1) {
      currentResults[index] = updatedQuestion;
    } else {
      currentResults.push(updatedQuestion);
    }
    this.results$.next([...currentResults]);
  }
}
